//  FindDialog.cpp version 1.5
//  yudit package - Unicode Editor for the X Window System (and Linux) 
//
//  Author: gsinai@iname.com (Gaspar Sinai)
//  GNU Copyright (C) 1997,1998,1999  Gaspar Sinai
// 
//  yudit version 1.5  Copyright(C) 30 November, 1999, Tokyo Japan  Gaspar Sinai
//  yudit version 1.4  Copyright(C) 25 November, 1999, Tokyo Japan  Gaspar Sinai
//  yudit version 1.3  Copyright(C)  5 April,    1999, Tokyo Japan  Gaspar Sinai
//  yudit version 1.2  Copyright(C) 10 December, 1998, Tokyo Japan  Gaspar Sinai
//  yudit version 1.1  Copyright(C) 23 August,   1998, Tokyo Japan  Gaspar Sinai
//  yudit version 1.0  Copyright(C) 17 May,      1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.99 Copyright(C)  4 April,    1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.97 Copyright(C)  4 February, 1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.95 Copyright(C) 10 January,  1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.94 Copyright(C) 17 December, 1997, Tokyo Japan  Gaspar Sinai
//  yudit version 0.9 Copyright (C)  8 December, 1997, Tokyo Japan  Gaspar Sinai
//  yutex version 0.8 Copyright (C)  5 November, 1997, Tokyo Japan  Gaspar Sinai
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

#include <FindDialog.h>
#include <UFontFreeType.h>
#include <Translate.h>
#include <stdlib.h>

FindDialog::FindDialog (QWidget* parent, QUWidgetBase* quwidgetIn, const Config* config) :
        QDialog (parent, "Yudit: Find", TRUE)
{
	const XInputMap*	xInputMap;
	const char*		xInputName;

	topConfig = config;
	uWidgetParent = quwidgetIn;
	layout = new QGridLayout (this, 5, 3, 4);
	layout->setRowStretch (0, 1);
	layout->setRowStretch (1, 1);
	layout->setRowStretch (2, 1);
	layout->setRowStretch (3, 1);
	layout->setRowStretch (4, 1);

	// Create the find area

	replaceLabel = new QLabel (this, "Replace With:");
	replaceLabel->setText (Translate("Replace With:"));
	replaceLabel->setMargin(2);
	replaceLabel->setAlignment (AlignRight|AlignVCenter);

	replaceXLabel = new QLabel (this, "X Area:");
	replaceXLabel->setText ("");
	replaceXLabel->setMargin(2);
	replaceXLabel->setAlignment (AlignRight|AlignVCenter);

	replaceWidget = new QUEntry (this, 
			(const char*)config->getChecked(Config::FONTNAME));

	replaceWidget->getWidget()->uInherit (uWidgetParent);
	replaceWidget->setMultiLine (UWidget::SINGLE);

	replaceWidget->setBackground ("white");
	replaceWidget->setForeground ("black");
	replaceWidget->setCursorColor ("black");

	findLabel = new QLabel (this, "Find String:");
	findLabel->setText (Translate("Find String:"));
	findLabel->setMargin(2);
	findLabel->setAlignment (AlignRight|AlignVCenter);

	findXLabel = new QLabel (this, "X Area:");
	findXLabel->setText ("");
	findXLabel->setMargin(2);
	findXLabel->setAlignment (AlignRight|AlignVCenter);

	findWidget = new QUEntry (this, (const char*)config->getChecked(Config::FONTNAME));
	findWidget->getWidget()->uInherit (replaceWidget->getWidget());
	findWidget->setEditable (UWidget::EDITABLE);

	findWidget->setBackground ("white");
	findWidget->setForeground ("black");
	findWidget->setCursorColor ("black");

	xInputName = (const char*) config->getChecked (Config::XINPUT);
	xInputMap = config->getXInputMap (xInputName);
	if (xInputMap==0)
	{
		cerr << "error: can not get XInputMap '"
			<< xInputName << "'.\n";
	}
	else
	{
		replaceWidget->uSetXInput (xInputName,
			(const char*) xInputMap->list[XInputMap::TYPE],
			(const char*) xInputMap->list[XInputMap::STYLE],
			(const char*) xInputMap->list[XInputMap::TEXT_TYPE],
			(const char*) xInputMap->list[XInputMap::ENCODING],
			(const char*) xInputMap->list[XInputMap::FONTMAP],
			atof ((const char*)xInputMap->list[XInputMap::VERSION]));
		findWidget->uSetXInput (xInputName,
			(const char*) xInputMap->list[XInputMap::TYPE],
			(const char*) xInputMap->list[XInputMap::STYLE],
			(const char*) xInputMap->list[XInputMap::TEXT_TYPE],
			(const char*) xInputMap->list[XInputMap::ENCODING],
			(const char*) xInputMap->list[XInputMap::FONTMAP],
			atof ((const char*)xInputMap->list[XInputMap::VERSION]));
	}
	replaceWidget->setMinimumSize (quwidgetIn->uMinimumWidth()*6,
		quwidgetIn->uMinimumHeight());
	findWidget->setMinimumSize (quwidgetIn->uMinimumWidth()*6, 
		quwidgetIn->uMinimumHeight());


	buttonFind = new QPushButton (Translate ("Find"), this);
	buttonFind->setMinimumSize (buttonFind->sizeHint());

	buttonFindReplace = new QPushButton (Translate ("Find/Replace"), this);
	buttonFindReplace->setMinimumSize (buttonFindReplace->sizeHint());

	buttonDismiss = new QPushButton (Translate ("Dismiss"), this);
	buttonDismiss->setMinimumSize (buttonDismiss->sizeHint());

	connect (buttonFind, SIGNAL (clicked()), SLOT (uFind()));
	connect (buttonFindReplace, SIGNAL (clicked()), SLOT (uFindReplace()));
	connect (buttonDismiss, SIGNAL (clicked()), SLOT (reject()));


	//
	// Layoutmanager bug.
	//
	if (replaceWidget->minimumSize().height() <= replaceLabel->sizeHint().height() * 2)
	{
		replaceLabel->setMinimumSize (replaceLabel->sizeHint());
		replaceXLabel->setMinimumSize (replaceLabel->sizeHint());
	}
	else
	{
		replaceLabel->setMinimumSize (replaceLabel->sizeHint().width(),
			replaceWidget->minimumSize().height()/2);
		replaceXLabel->setMinimumSize (replaceLabel->sizeHint().width(),
			replaceWidget->minimumSize().height()/2);
	}
	if (findWidget->minimumSize().height() <= findLabel->sizeHint().height() * 2)
	{
		findLabel->setMinimumSize (findLabel->sizeHint());
		findXLabel->setMinimumSize (findLabel->sizeHint());
	}
	else
	{
		findLabel->setMinimumSize (findLabel->sizeHint().width(),
			findWidget->minimumSize().height()/2);
		findXLabel->setMinimumSize (findLabel->sizeHint().width(),
			findWidget->minimumSize().height()/2);
	}
	if (findWidget->minimumSize().width() + 8 > 
		buttonFindReplace->minimumSize().width() + buttonDismiss->minimumSize().width())
	{
		buttonDismiss->setMinimumSize (findWidget->minimumSize().width() - buttonFindReplace->minimumSize().width()+8, 
			buttonDismiss->minimumSize().height());
	}
	layout->addWidget (findLabel, 0, 0, AlignRight|AlignVCenter);
	layout->addWidget (findXLabel, 1, 0, AlignRight|AlignVCenter);
	layout->addWidget (replaceLabel, 2, 0, AlignRight|AlignVCenter);
	layout->addWidget (replaceXLabel, 3, 0, AlignRight|AlignVCenter);
	layout->addMultiCellWidget (findWidget, 0, 1, 1, 2, AlignRight|AlignVCenter);
	layout->addMultiCellWidget (replaceWidget, 2, 3, 1, 2, AlignRight|AlignVCenter);

	layout->addWidget (buttonFind, 4, 0, AlignHCenter|AlignBottom);
	layout->addWidget (buttonFindReplace, 4, 1, AlignHCenter|AlignBottom);
	layout->addWidget (buttonDismiss, 4, 2, AlignHCenter|AlignBottom);

	findResult = -2;
	setCaption ((const char*) "Yudit: Find");
}

FindDialog::~FindDialog ()
{
	delete replaceLabel;
	delete replaceXLabel;
	delete replaceWidget;

	delete findLabel;
	delete findXLabel;
	delete findWidget;

	delete buttonFind;
	delete buttonFindReplace;
	delete buttonDismiss;
	delete layout;
}

void
FindDialog::findString ()
{
	// Layoutmanager bug.
	layout->activate();
	resize (sizeHint().width()+6, sizeHint().height()+6);
	exec();
	return;
}

void
FindDialog::uFind ()
{
	const UCS2*  const* textBuffer;
	const int*   textSizes;

	textBuffer = findWidget->uGetTextBuffer();
	textSizes = findWidget->uGetTextSizes();
	if (textBuffer!=0 && textSizes[0] !=0)
	{
		findResult = uWidgetParent->uFindUCS2 (textBuffer[0],
			textSizes[0], (findResult==-1) ? 1 : 0);
	}
	else
	{
		findResult = -1;
	}
}

void
FindDialog::uFindReplace ()
{
	const UCS2* const* textBuffer;
	const int*	textSizes;

	if (findResult<0) uFind();

	textBuffer = replaceWidget->uGetTextBuffer();
	textSizes = replaceWidget->uGetTextSizes();

	if (findResult<0) return;

	if (textBuffer==0)
	{
		uWidgetParent->uReplaceUCS2 (0, 0);
	}
	else
	{
		uWidgetParent->uReplaceUCS2 (textBuffer[0], textSizes[0]);
	}
	findResult = -2;
}
