//  MainWidget.h version 1.5
//  yudit package - Unicode Editor for the X Window System (and Linux) 
//
//  Author: gsinai@iname.com (Gaspar Sinai)
//  GNU Copyright (C) 1997,1998,1999  Gaspar Sinai
// 
//  yudit version 1.5  Copyright(C) 30 November, 1999, Tokyo Japan  Gaspar Sinai
//  yudit version 1.4  Copyright(C) 25 November, 1999, Tokyo Japan  Gaspar Sinai
//  yudit version 1.3  Copyright(C)  5 April,    1999, Tokyo Japan  Gaspar Sinai
//  yudit version 1.2  Copyright(C) 10 December, 1998, Tokyo Japan  Gaspar Sinai
//  yudit version 1.1  Copyright(C) 23 August,   1998, Tokyo Japan  Gaspar Sinai
//  yudit version 1.0  Copyright(C) 17 May,      1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.99 Copyright(C)  4 April,    1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.97 Copyright(C)  4 February, 1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.95 Copyright(C) 10 January,  1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.94 Copyright(C) 17 December, 1997, Tokyo Japan  Gaspar Sinai
//  yudit version 0.9 Copyright (C)  8 December, 1997, Tokyo Japan  Gaspar Sinai
//  yutex version 0.8 Copyright (C)  5 November, 1997, Tokyo Japan  Gaspar Sinai
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

#ifndef __main_widget_h_
#define __main_widget_h_

#include <X11/X.h>
#include <qlayout.h>
#include "PrintDialog.h"
#include <UXInput.h>
#include "FindDialog.h"

//
// X.h defines them.
//
#ifdef TrueColor
# undef TrueColor
#endif
#ifdef Unsorted
# undef Unsorted
#endif

#include <qwidget.h>
#include <qlayout.h> 
#include <qmessagebox.h>
#include <qevent.h>
#include <AString.h>

#include <qfiledialog.h>
#include <qevent.h>
#include <unistd.h>

#ifdef WITH_KDE

#include <kmenubar.h>
#include <kapp.h>
#include <ktopwidget.h>

#define YApplication KApplication

#else

#define KMenuBar QMenuBar
#define KTopLevelWidget QWidget

#include <qmenubar.h>
#include <qapp.h>
#include <qframe.h>

#define YApplication QApplication


#define KDNDDropZone int

#endif

#include <MenuBar.h>
#include <Config.h>
#include <UFont.h>
#include <UFontFreeType.h>
#include <QUWidget.h>
#include <qtimer.h>

class KMenuBarAction : public KMenuBar, public MenuBarAction
{
public:
			KMenuBarAction (QWidget* prnt);
			~KMenuBarAction ();
	virtual int	aInsert (const char* text, QPopupMenu* menu);
};

class MyTopLevelWidget : public KTopLevelWidget
{
public:					MyTopLevelWidget (QObject* parentIn);
					~MyTopLevelWidget();
	void				closeEvent (QCloseEvent *e);
private:
	QObject*			parent;
};

class MainWidget : public QObject 
{
	Q_OBJECT
public:
					MainWidget (const Config& configIn, 
						const char* fileNameIn=0,
						const char* encodingIn=0,
						const UWidget::UEditable=UWidget::EDITABLE);
					~MainWidget ();

	int				appendFile (const char* filenameIn);
	int				saveFile (const char* filenameIn);
	int				loadFile (const char* filenameIn);
	
	void				closeEvent (QCloseEvent *e);
	const Config&			getConfig()
					{ return config ; };
protected:
	MyTopLevelWidget*		topLevelWidget;
	Config				config;
	MenuBar*			menuBar;
	KMenuBarAction*			menuBarAction;
	static QList<MainWidget> 	windowList;
	QTimer*				destroyTimer;			
private slots:
	void				destroySlot();
	void				changeConfigSlot (
						Config::CheckList type,
						const char* setTo);
	void				doPrintSlot();
	void				helpSlot (MenuBar::HelpType type);
	void				searchSlot (MenuBar::SearchType type);

	void 				newMainWidget();
	void 				quitMainWidget();
	void 				exitMainWidget();

	void				editConfig();
	void				saveAs();
	void				save();
	void				loadFile();
	void				appendFile();
	void				reloadFile();
	void				dropEvent (KDNDDropZone *dropZone);
private:
	void				setupMainWidget (const char* filenameIn=0);
	int				setXInput (const char* name);

	UWidget::UEditable		editableFlag;
	QFileDialog*			fileDialog;
	QFileDialog*			saveDialog;
	PrintDialog*			printDialog;
	FindDialog*			findDialog;
	QUWidget*			editArea;
	QBoxLayout*			layout;
	AString				fileName;
	AString				encoding;
	AString				readDir;
	AString				writeDir;
	KDNDDropZone*			dropZone;
	int				checkEncoding();
	int				appendFileBase (const char* filenameIn);
};

#endif /* __main_widget_h_ */
